@@ -1,22 +1,27 @@
 from typing import Any, Dict, List, Tuple, Union
-from pathlib import Path
 
+from collections import namedtuple
 import json
 import os
 
 from dash import dash_table, dcc, html
 from dash.dependencies import Input, Output, State
 from dash.exceptions import PreventUpdate
-from flask_healthz import healthz
 import dash
-import flasgger as swg
 import flask
+from flask import current_app
 import flask_monitoringdashboard as fmdb
+import requests
 
-import hidebound.server.api as api
-import hidebound.server.components as components
-import hidebound.server.server_tools as server_tools
 from hidebound.core.config import Config
+import hidebound.server.components as components
+import hidebound.server.extensions as ext
+import hidebound.server.server_tools as server_tools
+
+
+TESTING = True
+if __name__ == '__main__':
+    TESTING = False  # pragma: no cover
 # ------------------------------------------------------------------------------
 
 
@@ -25,6 +30,24 @@ Hidebound service used for displaying and interacting with Hidebound database.
 '''
 
 
+HOST = '0.0.0.0'
+PORT = 8080
+Endpoints = namedtuple(
+    'Endpoints',
+    ['api', 'init', 'update', 'create', 'export', 'delete', 'read', 'search'],
+)
+EP = Endpoints(
+    api=f'http://{HOST}:{PORT}/api',
+    init=f'http://{HOST}:{PORT}/api/initialize',
+    update=f'http://{HOST}:{PORT}/api/update',
+    create=f'http://{HOST}:{PORT}/api/create',
+    export=f'http://{HOST}:{PORT}/api/export',
+    delete=f'http://{HOST}:{PORT}/api/delete',
+    read=f'http://{HOST}:{PORT}/api/read',
+    search=f'http://{HOST}:{PORT}/api/search',
+)
+
+
 def liveness():
     # type: () -> None
     '''Liveness probe for kubernetes.'''
@@ -39,8 +62,8 @@ def readiness():
     pass
 
 
-def get_app():
-    # type: () -> dash.Dash
+def get_app(testing=False):
+    # type: (bool) -> dash.Dash
     '''
     Creates a Hidebound app.
 
@@ -48,15 +71,17 @@ def get_app():
         Dash: Dash app.
     '''
     app = flask.Flask('hidebound')  # type: Union[flask.Flask, dash.Dash]
-    swg.Swagger(app)
-    app.register_blueprint(api.API)
+    app.config.update(
+        TESTING=testing,
+        HEALTHZ=dict(
+            live=liveness,
+            ready=readiness,
+        )
+    )
 
-    # healthz endpoints
-    app.register_blueprint(healthz, url_prefix="/healthz")
-    app.config.update(HEALTHZ={
-        "live": liveness,
-        "ready": readiness,
-    })
+    ext.swagger.init_app(app)
+    ext.hidebound.init_app(app)
+    ext.healthz.init_app(app)
 
     # flask monitoring
     fmdb.config.link = 'monitor'
@@ -68,8 +93,7 @@ def get_app():
     return app
 
 
-APP = get_app()
-CONFIG_PATH = None  # type: Union[str, Path, None]
+APP = get_app(testing=TESTING)
 
 
 @APP.server.route('/static/<stylesheet>')
@@ -122,9 +146,10 @@ def on_event(*inputs):
         dict: Store data.
     '''
     APP.logger.debug(f'on_event called with inputs: {str(inputs)[:50]}')
+    hb = current_app.extensions['hidebound']
 
     store = inputs[-1] or {}  # type: Any
-    config = store.get('config', api.CONFIG)  # type: Dict
+    config = store.get('config', hb.config)  # type: Dict
     conf = json.dumps(config)
 
     context = dash.callback_context
@@ -144,42 +169,41 @@ def on_event(*inputs):
 
     input_id = context.triggered[0]['prop_id'].split('.')[0]
 
-    server = APP.server
     if input_id == 'init-button':
-        response = server.test_client().post('/api/initialize', json=conf).json
+        response = requests.post(EP.init, json=conf).json()
         if 'error' in response.keys():
             store['/api/read'] = response
 
     elif input_id == 'update-button':
-        if api.DATABASE is None:
-            response = server.test_client().post('/api/initialize', json=conf).json
+        if hb.database is None:
+            response = requests.post(EP.init, json=conf).json()
             if 'error' in response.keys():
                 store['/api/read'] = response
 
-        server.test_client().post('/api/update')
+        requests.post(EP.update)
 
         params = json.dumps({'group_by_asset': grp})
-        response = server.test_client().post('/api/read', json=params).json
+        response = requests.post(EP.read, json=params).json()
         store['/api/read'] = response
 
     elif input_id == 'create-button':
-        server.test_client().post('/api/create')
+        requests.post(EP.create)
 
     elif input_id == 'export-button':
-        response = server.test_client().post('/api/export')
+        response = requests.post(EP.export)
         code = response.status_code
         if code < 200 or code >= 300:
             store['/api/read'] = response.json
 
     elif input_id == 'delete-button':
-        server.test_client().post('/api/delete')
+        requests.post(EP.delete)
 
     elif input_id == 'search-button':
         query = json.dumps({
             'query': inputs_['query'],
             'group_by_asset': inputs_['dropdown']
         })
-        response = server.test_client().post('/api/search', json=query).json
+        response = requests.post(EP.search, json=query).json()
         store['/api/read'] = response
         store['query'] = inputs_['query']
 
@@ -196,15 +220,15 @@ def on_event(*inputs):
             store['config'] = temp
             store['config_error'] = server_tools.error_to_response(error).json
 
-    elif input_id == 'write-button':
-        try:
-            config = store['config']
-            Config(config).validate()
-            with open(CONFIG_PATH, 'w') as f:  # type: ignore
-                json.dump(config, f, indent=4, sort_keys=True)
-            store['config_error'] = None
-        except Exception as error:
-            store['config_error'] = server_tools.error_to_response(error).json
+    # elif input_id == 'write-button':
+    #     try:
+    #         config = store['config']
+    #         Config(config).validate()
+    #         with open(CONFIG_PATH, 'w') as f:  # type: ignore
+    #             json.dump(config, f, indent=4, sort_keys=True)
+    #         store['config_error'] = None
+    #     except Exception as error:
+    #         store['config_error'] = server_tools.error_to_response(error).json
 
     return store
 
@@ -256,6 +280,8 @@ def on_get_tab(tab, store):
     Returns:
         flask.Response: Response.
     '''
+    hb = current_app.extensions['hidebound']
+
     APP.logger.debug(
         f'on_get_tab called with tab: {tab} and store: {str(store)[:50]}'
     )
@@ -277,7 +303,7 @@ def on_get_tab(tab, store):
         return components.get_asset_graph(data['response'])
 
     elif tab == 'config':
-        config = store.get('config', api.CONFIG)
+        config = store.get('config', hb.config)
         return components.get_config_tab(config)
 
     elif tab == 'api':
@@ -336,19 +362,4 @@ def on_config_card_update(timestamp, store):
 
 if __name__ == '__main__':
     debug = 'DEBUG_MODE' in os.environ.keys()
-    config_path = None  # type: Any
-    if debug:
-        config, config_path = server_tools\
-            .setup_hidebound_directory(
-                '/tmp',
-                config_path='/home/ubuntu/hidebound/resources/test_config.json'
-            )
-    else:
-        config_path = '/mnt/storage/hidebound/config/hidebound_config.json'
-        if not Path(config_path).is_file():
-            config_path = None
-        config, config_path = server_tools \
-            .setup_hidebound_directory('/mnt/storage', config_path=config_path)
-    api.CONFIG = config
-    CONFIG_PATH = config_path
-    APP.run_server(debug=debug, host='0.0.0.0', port=8080)
+    APP.run_server(debug=debug, host=HOST, port=PORT)
